# -*- coding: utf-8 -*-
from trac.core import *
from trac.wiki.api import IWikiMacroProvider
from StringIO import StringIO
import inspect

__all__ = ['AutoNavMacro']

class AutoNavMacro(Component):
    """ AutoNav finds all references in the wiki section to this Document
	and shows them in a sorted list.

	Used with no arguments only produces a list from the database. Arguments
	sent to AutoNav will be merged inside the list too. Separate the
	arguments with comma.

	Example:
	    {{{[[AutoNav()]]}}}	-> only references
	    
	    {{{[[AutoNav(MyPage)]]}}} -> references merged and sorted with MyPage
	    
	    {{{[[AutoNav(MyPage, MyPageToo, MyPageThree)]]}}} -> references merged with MyPage, MyPageToo and MyPageThree
    """
    
    implements(IWikiMacroProvider)
    
    # IWikiMacroProvider methods
    def get_macros(self):
        yield 'AutoNav'
        
    def get_macro_description(self, name):
        return inspect.getdoc(self.__class__)

    def render_macro(self, req, name, args):
	db = self.env.get_db_cnx()
	cursor = db.cursor()
	buf = StringIO()
	
	# get the refere page name
	thispage = req.path_info.startswith('/wiki/') \
	           and req.path_info[6:] or 'WikiStart'
	
	# process arguments
	pages = []
	if args:
	    pages = [page.strip() for page in args.split(',')]
	
	# query to get the latest version of a page
	query = """
	    SELECT w1.name 
	    FROM wiki w1, 
	        ( 
		    SELECT name, MAX(version) AS version 
		    FROM wiki 
		    GROUP BY name 
		) w2 
	    WHERE 
	        w1.version = w2.version AND 
		w1.name = w2.name AND 
		w1.text LIKE \'%%%s%%\' 
	    ORDER BY w1.name""" % thispage
	
	# TODO: use named parameters
	cursor.execute(query)
	
	# for each answer store in page
	while 1:
	    row = cursor.fetchone()
	    if row == None:
	        break
	    if row[0] == thispage:
	        continue
	    pages.append(row[0])
	
	pages.sort()
	
	# get the references to each list
	outlist = []
	for page in pages:
	    out = '<a href="%s">%s</a>' % (self.env.href.wiki(page), page)
	    outlist.append(out)
	
	# format the list
	buf.write('<strong>Navigation:</strong> (')
	buf.write(", ".join(outlist))
	buf.write(')\n')
	
	return buf.getvalue()
